import holoviews as hv
import numpy as np

from .axis import Axis
from .figure import Figure


class Bars(Figure):
    def __init__(
        self,
        counts: list | np.ndarray,
        ticks: list[str],
        title: str | None = None,
        axis_x: str | Axis | dict = "X",
        axis_y: str | Axis | dict = "Y",
        width: int | None = None,
        height: int | None = None,
        color: str | list[str] = "blue",
        labels: list[str] | None = None,
    ):
        # inheritances
        Figure.__init__(
            self,
            title=title,
            width=width,
            height=height,
        )

        self.counts: list | np.ndarray = counts
        self.ticks: list[str] = ticks
        self.labels: list[str] | None = labels

        self.axis_x = Axis.init(axis_x)
        self.axis_y = Axis.init(axis_y)
        self.color: str | list[str] = color

        return None

    def to_holoviews(self) -> hv.Overlay:
        bars = []
        for i in range(len(self.counts)):
            bars.append(
                hv.Bars(
                    [(self.ticks[i], self.counts[i])],
                    self.axis_x.hv_dimension,
                    vdims=[self.axis_y.hv_dimension],
                    label=self.labels[i] if self.labels is not None else None,
                ).opts(
                    color=self.color[i] if self.color is not None else None,
                )
            )

        return hv.Overlay(bars).opts(
            width=self.width,
            height=self.height,
            title=self.title,
        )



from __future__ import annotations

import holoviews as hv


class Axis:
    def __init__(
        self,
        title: str,
        id: str = "",
        unit: str | None = None,
        shared: bool = False,
    ):
        self.id = id
        self.title = title
        self.unit = unit
        self.shared = shared
        # self.range
        return None

    @property
    def label(self) -> str:
        if self.unit is None:
            return self.title

        return f"{self.title} [{self.unit}]"

    @property
    def hv_dimension(self) -> hv.Dimension:
        return hv.Dimension(self.id, label=self.label)

    @classmethod
    def init(cls, input: str | dict | Axis) -> Axis:
        if isinstance(input, Axis):
            return input
        elif isinstance(input, str):
            return Axis(id=input, title=input)
        elif isinstance(input, dict):
            return Axis(**input)
        else:
            raise ValueError(f"Invalid axis type: {type(input)}")



import holoviews as hv
import numpy as np
from bokeh.plotting import show

from .axis import Axis
from .datavector import DataVector
from .figure import Figure

hv.extension("bokeh")


class Curve(Figure):
    def __init__(
        self,
        data_x: list | np.ndarray | None = None,
        data_y: list | np.ndarray | None = None,
        title: str | None = None,
        axis_x: str | Axis | dict = "X",
        axis_y: str | Axis | dict = "Y",
        width: int | None = None,
        height: int | None = None,
        color: str = "blue",
        label: str = "",
        interpolation: str = "linear",
    ):
        if interpolation not in ["linear", "ffill", "bfill"]:
            raise ValueError(
                f"Interpolation method '{interpolation}' is not supported."
            )
        # inheritances
        Figure.__init__(
            self,
            title=title,
            width=width,
            height=height,
        )

        self.data_x: list | np.ndarray | None = data_x
        self.data_y: list | np.ndarray | None = data_y
        self.axis_x = Axis.init(axis_x)
        self.axis_y = Axis.init(axis_y)
        self.color: str = color
        self.label: str = label

        self.interpolation: str = interpolation

        return None

    def to_holoviews(self) -> hv.Scatter:
        ret = hv.Curve(
            (self.data_x, self.data_y),
            kdims=[self.axis_x.hv_dimension],
            vdims=[self.axis_y.hv_dimension],
            label=self.label,
        ).opts(
            width=self.width,
            height=self.height,
            title=self.title,
            color=self.color,
            shared_axes=False,
            interpolation=interpolation_to_holoviews(interpolation=self.interpolation),
        )

        return ret


def interpolation_to_holoviews(interpolation: str) -> str:
    if interpolation == "linear":
        return "linear"
    elif interpolation == "ffill":
        return "steps-post"
    elif interpolation == "bfill":
        return "steps-pre"
    else:
        raise ValueError(f"Interpolation method '{interpolation}' is not supported.")


class CurveDataVector(Figure):
    def __init__(
        self,
        data_vector: DataVector | list[DataVector],
        title: str | None = None,
        axis_x: str | Axis | dict = "X",
        axis_y: str | Axis | dict = "Y",
        width: int | None = None,
        height: int | None = None,
    ):
        # inheritances
        Figure.__init__(
            self,
            title=title,
            width=width,
            height=height,
        )

        self.axis_x = Axis.init(axis_x)
        self.axis_y = Axis.init(axis_y)

        if not isinstance(data_vector, list):
            data_vector = [data_vector]
        self.data_vector = data_vector

        return None

    def to_holoviews(self) -> hv.Overlay:
        ret = []

        for data_vector in self.data_vector:
            ret.append(
                hv.Curve(
                    (data_vector.data_x, data_vector.data_y),
                    kdims=[self.axis_x.hv_dimension],
                    vdims=[self.axis_y.hv_dimension],
                    label=data_vector.label,
                ).opts(
                    width=self.width,
                    height=self.height,
                    title=self.title,
                    color=data_vector.color,
                    shared_axes=False,
                    interpolation=interpolation_to_holoviews(
                        interpolation=data_vector.interpolation
                    ),
                )
            )

        return hv.Overlay(ret)

    def show_holoviews(self):
        return show(hv.render(self.to_holoviews()))



from .axis import Axis
from .bars import Bars
from .confusion_matrix import ConfusionMatrix
from .curve import Curve, CurveDataVector
from .datavector import DataVector
from .figure import Figure
from .hist import Histogram
from .scatter import Scatter



import holoviews as hv
import numpy as np

from .axis import Axis
from .figure import Figure

hv.extension("bokeh")


class Scatter(Figure):
    def __init__(
        self,
        data_x: list | np.ndarray | None = None,
        data_y: list | np.ndarray | None = None,
        title: str | None = None,
        axis_x: str | Axis | dict = "X",
        axis_y: str | Axis | dict = "Y",
        width: int | None = None,
        height: int | None = None,
        color: str = "blue",
        label: str = "",
    ):
        # inheritances
        Figure.__init__(
            self,
            title=title,
            width=width,
            height=height,
        )

        self.data_x: list | np.ndarray | None = data_x
        self.data_y: list | np.ndarray | None = data_y
        self.axis_x = Axis.init(axis_x)
        self.axis_y = Axis.init(axis_y)
        self.color: str = color
        self.label: str = label

        return None

    def to_holoviews(self) -> hv.Scatter:
        ret = hv.Scatter(
            (self.data_x, self.data_y),
            kdims=[self.axis_x.hv_dimension],
            vdims=[self.axis_y.hv_dimension],
            label=self.label,
        ).opts(
            width=self.width,
            height=self.height,
            title=self.title,
            color=self.color,
            shared_axes=False,
        )

        return ret



import numpy as np


class DataVector:
    def __init__(
        self,
        data_x: list | np.ndarray | None = None,
        data_y: list | np.ndarray | None = None,
        color: str = "blue",
        label: str = "",
        interpolation: str = "linear",
    ):
        if interpolation not in ["linear", "ffill", "bfill"]:
            raise ValueError(
                f"Interpolation method '{interpolation}' is not supported."
            )
        self.data_x: list | np.ndarray | None = data_x
        self.data_y: list | np.ndarray | None = data_y

        self.color: str = color
        self.label: str = label
        self.interpolation: str = interpolation

        return None
